﻿Option Strict On
Imports Microsoft.Kinect
Imports System.IO
Class MainWindow

    Const ScreenWidth As Single = 640.0F
    Const ScreenHeight As Single = 480.0F
    Const 関節の太さ As Double = 3
    Const ボディセンターの太さ As Double = 10

    Dim centerPointBrush As Brush = Brushes.Blue
    Dim 追跡された関節のブラシ As Brush = New SolidColorBrush(Color.FromArgb(255, 68, 192, 68))
    Dim 推測された関節のブラシ As Brush = Brushes.Yellow
    Dim 追跡された骨のペンの色と幅 As New Pen(Brushes.Red, 6)
    Dim inferredBonePen As New Pen(Brushes.Navy, 10)
    Dim newSensor As KinectSensor
    Dim myDrawingGroup As DrawingGroup
    Dim imageSource As DrawingImage


    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        myDrawingGroup = New DrawingGroup()
        imageSource = New DrawingImage(myDrawingGroup)
        stickManImage.Source = imageSource
        AddHandler KinectSensorChooser1.KinectSensorChanged, Sub(kinectSender As Object, kinectArgs As DependencyPropertyChangedEventArgs)
                                                                 Dim oldSensor = DirectCast(kinectArgs.OldValue, KinectSensor)
                                                                 If oldSensor Is Nothing = False Then
                                                                     StopKinect(oldSensor)
                                                                 End If

                                                                 newSensor = DirectCast(kinectArgs.NewValue, KinectSensor)
                                                                 If newSensor Is Nothing = True Then
                                                                     Return
                                                                 End If
                                                                 newSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
                                                                 AddHandler newSensor.SkeletonFrameReady, AddressOf SkeletonsReady
                                                                 newSensor.SkeletonStream.Enable(New TransformSmoothParameters() With {.Smoothing = 0.5F,
                                                                                                                                       .Correction = 0.5F,
                                                                                                                                       .Prediction = 0.5F,
                                                                                                                                       .JitterRadius = 0.05F,
                                                                                                                                       .MaxDeviationRadius = 0.04F})

                                                                 Try
                                                                     newSensor.Start()
                                                                 Catch
                                                                     MessageBox.Show("KINECTが接続されておりません。")
                                                                     Exit Sub
                                                                 End Try


                                                             End Sub
    End Sub

    Private Sub SkeletonsReady(ByVal sender As Object, ByVal e As SkeletonFrameReadyEventArgs)
        Dim skeletons(-1) As Skeleton

        Using skeletonFrame As SkeletonFrame = e.OpenSkeletonFrame()
            If skeletonFrame Is Nothing = False Then
                skeletons = New Skeleton(skeletonFrame.SkeletonArrayLength - 1) {}
                skeletonFrame.CopySkeletonDataTo(skeletons)
            End If
        End Using

        Using myDrawingContext As DrawingContext = myDrawingGroup.Open()
            myDrawingContext.DrawRectangle(Brushes.Transparent, Nothing, New Rect(0.0, 0.0, ScreenWidth, ScreenHeight))

            If skeletons.Length <> 0 Then
                For Each mySkeleton As Skeleton In skeletons
                    If mySkeleton.TrackingState = SkeletonTrackingState.Tracked Then
                        DrawBonesAndJoints(mySkeleton, myDrawingContext)
                    ElseIf mySkeleton.TrackingState = SkeletonTrackingState.PositionOnly Then
                        myDrawingContext.DrawEllipse(centerPointBrush, Nothing, SkeletonPointToScreen(mySkeleton.Position), ボディセンターの太さ, ボディセンターの太さ)
                    End If
                Next
            End If
        End Using
    End Sub

    Private Sub DrawBonesAndJoints(ByVal skeleton As Skeleton, ByVal drawingContext As DrawingContext)
        '胴体
        DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter)
        DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft)
        DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight)
        DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine)
        DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter)
        DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft)
        DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight)

        '左腕
        DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft)
        DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft)
        DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft)

        ' 右腕
        DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight)
        DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight)
        DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight)

        '左脚
        DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft)
        DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft)
        DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft)

        ' 右脚
        DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight)
        DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight)
        DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight)

        For Each myJoint As Joint In skeleton.Joints
            Dim drawBrush As Brush = Nothing

            If myJoint.TrackingState = JointTrackingState.Tracked Then
                drawBrush = 追跡された関節のブラシ
            ElseIf myJoint.TrackingState = JointTrackingState.Inferred Then
                drawBrush = 推測された関節のブラシ
            End If

            If drawBrush Is Nothing = False Then
                drawingContext.DrawEllipse(drawBrush, Nothing, SkeletonPointToScreen(myJoint.Position), 関節の太さ, 関節の太さ)
            End If
        Next
    End Sub

    Private Function SkeletonPointToScreen(ByVal skelpoint As SkeletonPoint) As Point
        Dim depthPoint As DepthImagePoint = newSensor.MapSkeletonPointToDepth(skelpoint, DepthImageFormat.Resolution640x480Fps30)

        Dim x As Double = depthPoint.X
        If x > ScreenWidth Then
            x = ScreenWidth
        ElseIf x < 0 Then
            x = 0
        End If

        Dim y As Double = depthPoint.Y
        If y > ScreenHeight Then
            y = ScreenHeight
        ElseIf y < 0 Then
            y = 0
        End If

        Return New Point(x, y)
    End Function

    Private Sub DrawBone(ByVal skeleton As Skeleton, ByVal drawingContext As DrawingContext, ByVal jointType0 As JointType, ByVal jointType1 As JointType)
        Dim myJoint0 As Joint = skeleton.Joints(jointType0)
        Dim myJoint1 As Joint = skeleton.Joints(jointType1)

        If myJoint0.TrackingState = JointTrackingState.NotTracked OrElse myJoint1.TrackingState = JointTrackingState.NotTracked Then
            Return
        End If

        If myJoint0.TrackingState = JointTrackingState.Inferred AndAlso myJoint1.TrackingState = JointTrackingState.Inferred Then
            Return
        End If

        Dim drawPen As Pen = inferredBonePen
        If myJoint0.TrackingState = JointTrackingState.Tracked AndAlso myJoint1.TrackingState = JointTrackingState.Tracked Then
            drawPen = 追跡された骨のペンの色と幅
        End If

        drawingContext.DrawLine(drawPen, SkeletonPointToScreen(myJoint0.Position), SkeletonPointToScreen(myJoint1.Position))
    End Sub


    Private Sub StopKinect(sensor As KinectSensor)
        If sensor Is Nothing = False Then
            If sensor.IsRunning = True Then
                RemoveHandler sensor.SkeletonFrameReady, AddressOf SkeletonsReady
                sensor.Stop()
                sensor.Dispose()
            End If
        End If
    End Sub
End Class
